﻿$(function () {
    var allPlatforms = [
        {
            text: "Windows XP",
            icon: "images/os-icons/windows-xp.png"
        },
        {
            text: "Windows Vista",
            selected: true,
            icon: "images/os-icons/windows-vista.png"
        },
        {
            text: "Windows 7",
            icon: "images/os-icons/windows-7.png"
        },
        {
            text: "Windows 8",
            icon: "images/os-icons/windows-8.png"
        },
        {
            text: "Windows 8.1",
            icon: "images/os-icons/windows-81.png"
        }
    ];

    var allBrowsers = [
        {
            text: "Internet Explorer",
            icon: "images/browser-icons/ie.png"
        },
        {
            text: "Chrome",
            selected: true,
            icon: "images/browser-icons/chrome.png"
        },
        {
            text: "Firefox",
            icon: "images/browser-icons/firefox.png"
        },
        {
            text: "Opera",
            icon: "images/browser-icons/opera.png"
        },
        {
            text: "Safari",
            icon: "images/browser-icons/safari.png"
        }
    ];

    localStorageGet('browsers', function (items) {
        if (!items.browsers) {
            // create a default browser list
            //
            function range(start, end) {
                var ret = [];
                for (var i = start; i <= end; i++) {
                    ret.push(i);
                }
                return ret;
            }

            var browserVersions = {
                'winxp' : {
                    'ie' : [ 6, 7, { text: 8, selected : true } ],
                    'chrome' : ["canary"].concat(range(30, 42)).concat({ text : 43, selected : true }),
                    'firefox' : ["nightly"].concat(range(30, 37)).concat({ text: 38, selected : true }),
                    'safari' : [ '4.0', '5.0.5', { text: '5.1', selected: true } ],
                    'opera' : ["next"].concat(range(20, 28)).concat({ text: 29, selected : true })
                },
                'winvista' : {
                    'ie' : [ { text : 9, selected : true } ],
                    'chrome' : ["canary"].concat(range(1, 42)).concat({ text : 43, selected : true }),
                    'firefox' : [ "nightly", 3, 3.5, 3.6 ].concat(range(4, 37)).concat({ text : 38, selected : true }),
                    'safari' : [ '4.0', '5.0.5', { text: '5.1', selected: true } ],
                    'opera' : [ "next", 10, 10.5, 11, 11.5, 11.6, 12 ].concat(range(15, 28)).concat({ text : 29, selected : true })
                },
                'win7' : {
                    'ie' : [ { text: 11, selected : true } ],
                    'chrome' : ["canary"].concat(range(30, 42)).concat({ text : 43, selected : true }),
                    'firefox' : ["nightly"].concat(range(30, 37)).concat({ text: 38, selected : true }),
                    'safari' : [ '4.0', '5.0.5', { text: '5.1', selected: true } ],
                    'opera' : ["next"].concat(range(20, 28)).concat({ text: 29, selected : true })
                },
                'win8' : {
                    'ie' : [ { text: 10, selected : true } ],
                    'chrome' : ["canary"].concat(range(30, 42)).concat({ text : 43, selected : true }),
                    'firefox' : ["nightly"].concat(range(30, 37)).concat({ text: 38, selected : true }),
                    'safari' : [ '4.0', '5.0.5', { text: '5.1', selected: true } ],
                    'opera' : ["next"].concat(range(20, 28)).concat({ text: 29, selected : true })
                },
                'win8.1' : {
                    'ie' : [ { text: 11, selected : true } ],
                    'chrome' : ["canary"].concat(range(30, 42)).concat({ text : 43, selected : true }),
                    'firefox' : ["nightly"].concat(range(30, 37)).concat({ text: 38, selected : true }),
                    'safari' : [ '4.0', '5.0.5', { text: '5.1', selected: true } ],
                    'opera' : ["next"].concat(range(20, 28)).concat({ text: 29, selected : true })
                }
            };

            // cache this list and init UI
            localStorageSet({
                browsers : browserVersions,
                cacheTime : 0
            },
            function () {
                init(browserVersions)
            });
        }
        else {
            init(items.browsers);
        }
    });

    function init (browserVersions) {
        var platformDropdown = new Dropdown({
            width : 170,
            height : 40,
            data : allPlatforms, 
            callback : function (selected) {
                var platform_version = selected;
                platform_version = platformVersionToShortName(platform_version);

                var browser = $('#browser .dropdown-selected .text').text();
                browser = normalizeBrowserName(browser);

                var versions = browserVersions['win' + platform_version][browser].slice();
                versions.reverse();
                updateVersionsDropdown(versions);
            }
        });
        $('#platform').append(platformDropdown.create());

        var browserDropdown = new Dropdown({
            width : 170,
            height : 40,
            iconWidth : 32,
            iconHeight : 32,
            data : allBrowsers,
            callback : function (selected) {
                var browser = selected;
                browser = normalizeBrowserName(browser);

                var platform_version = $('#platform .dropdown-selected .text').text();
                platform_version = platformVersionToShortName(platform_version);

                var versions = browserVersions['win' + platform_version][browser].slice();
                versions.reverse();
                updateVersionsDropdown(versions);
            }
        });
        $('#browser').append(browserDropdown.create());

        function updateVersionsDropdown (versions) {
            versionDropdown.update(versions)
        }

        // chrome on vista is selected by default
        var chromeVersions = browserVersions['winvista']['chrome'].slice();
        chromeVersions.reverse();
        var versionDropdown = new Dropdown({
            width : 90,
            height : 40,
            center : true,
            data : chromeVersions
        });
        $('#version').append(versionDropdown.create());

        // update dropdownds with latest browser info
        function updateDropdowns (browsers, freshOrCached) {
            var browser = $('#browser .dropdown-selected .text').text();
            browser = normalizeBrowserName(browser);
            var platform_version = $('#platform .dropdown-selected .text').text();
            platform_version = platformVersionToShortName(platform_version);

            var versions = browsers['win' + platform_version][browser].slice();
            versions.reverse();
            updateVersionsDropdown(versions);
        }

        function updateBrowserList (browserList) {
            var list = {};

            var platforms = Object.keys(browserList.win);
            platforms.forEach(function (platform) {
                list['win' + platform] = {};

                var browsers = Object.keys(browserList.win[platform])
                browsers.forEach(function (browser) {
                    // put any browser versions with text (nightly, next, canary) at the first position
                    var orderedBrowserList = [];
                    browserList['win'][platform][browser].forEach(function (v) {
                        if (/^\d+/.test(v)) {
                            orderedBrowserList.push(v);
                        }
                        else {
                            orderedBrowserList.unshift(v);
                        }
                    });
                    // make the last item selected for Dropdown
                    var last = orderedBrowserList[orderedBrowserList.length - 1];
                    last = { text : last, selected : true };
                    orderedBrowserList[orderedBrowserList.length - 1] = last;

                    list['win' + platform][browser] = orderedBrowserList;
                });
            });

            return list;
        }

        // get latest browser info
        function getLatestBrowsers (cb) {
            $.get('https://www.browserling.com/browsers.json', function (data) {
                try {
                    var browsers = JSON.parse(data);
                    cb(browsers);
                }
                catch (e) {
                    cb(null);
                }
            }, "text");
        }

        localStorageGet(['browsers', 'cacheTime'], function (items) {
            if (items.browsers && items.cacheTime) {
                if (Date.now() - items.cacheTime > 3*3600*1000) {
                    getLatestBrowsers(function (browsers) {
                        if (browsers) {
                            var browserList = updateBrowserList(browsers);
                            localStorageSet({
                                browsers : browserList,
                                cacheTime : Date.now()
                            },
                            function () {
                                updateDropdowns(browserList, "fresh");
                            });
                        }
                    });
                }
                else {
                    updateDropdowns(items.browsers, "cached");
                }
            }
            else {
                getLatestBrowsers(function (browsers) {
                    if (browsers) {
                        var browserList = updateBrowserList(browsers);
                        localStorageSet({
                            browsers : browserList,
                            cacheTime : Date.now()
                        },
                        function () {
                            updateDropdowns(browserList, "fresh");
                        });
                    }
                });
            }
        });
    }

    // make footer www.browserling.com clickable
    $('#footer a').click(function (ev) {
        ev.preventDefault();

        var newUrl = "https://www.browserling.com";
        createTab(newUrl);
    });

    function platformVersionToShortName (platform_version) {
        if (platform_version == "Windows XP") {
            platform_version = "xp";
        }
        else if (platform_version == "Windows Vista") {
            platform_version = "vista";
        }
        else if (platform_version == "Windows 7") {
            platform_version = "7";
        }
        else if (platform_version == "Windows 8") {
            platform_version = "8";
        }
        else if (platform_version == "Windows 8.1") {
            platform_version = "8.1";
        }
        return platform_version;
    }

    function normalizeBrowserName (browser) {
        if (browser == "Internet Explorer") {
            browser = "ie";
        }
        else {
            browser = browser.toLowerCase();
        }
        return browser;
    }

    // make run work
    $('#run input').click(function () {
        var platform_name = 'win';
        var platform_version = $('#platform .dropdown-selected .text').text();
        platform_version = platformVersionToShortName(platform_version);

        var browser = $('#browser .dropdown-selected .text').text();
        browser = normalizeBrowserName(browser);

        var version = $('#version .dropdown-selected .text').text();
        var url = $('#url input').val();

        var newUrl = "https://www.browserling.com/browse/" +
            platform_name + "/" +
            platform_version + "/" +
            browser + "/" +
            version + "/" +
            encodeURIComponent(url);

        createTab(newUrl);
    });

    // fill url field with current tab's url
    getCurrentUrl(function (url) {
        $('#url input').val(url);
    });
});
