var ToggleButton = require('sdk/ui/button/toggle').ToggleButton;
var panels = require("sdk/panel");
var self = require("sdk/self");
var tabs = require("sdk/tabs");
var ss = require("sdk/simple-storage");

var button = ToggleButton({
    id: "browserling",
    label: "Browserling",
    icon: {
        "16": "./icons/icon-16x16.png",
        "18": "./icons/icon-18x18.png",
        "32": "./icons/icon-32x32.png",
        "36": "./icons/icon-36x36.png",
        "64": "./icons/icon-64x64.png"
    },
    onClick: function (state) {
        if (state.checked) {
            var panel = panels.Panel({
                width: 470,
                height: 470,
                contentURL: self.data.url("popup.html"),
                onHide: function () {
                    button.state('window', { checked: false });
                },
                onShow: function () {
                    panel.port.emit('currentTabUrl', tabs.activeTab.url);
                }
            });

            panel.port.on('createTab', function (url) {
                tabs.open(url);
                panel.hide();
            });

            panel.port.on('localStorageSet', function (key, val) {
                ss.storage[key] = val;
            });

            panel.port.on('localStorageClear', function () {
                var keys = Object.keys(ss.storage);
                keys.forEach(function (key) {
                    delete ss.storage[key];
                });
            });

            panel.port.on('localStorageGet', function (eventId, items) {
                var responseEvent = 'localStorageGet-' + eventId;
                if (Object.prototype.toString.call(items) == "[object Array]") {
                    var returnItems = {};
                    items.forEach(function (key) {
                        if (key in ss.storage) {
                            var val = ss.storage[key];
                            returnItems[key] = val;
                        }
                    });
                    panel.port.emit(responseEvent, returnItems);
                }
                else {
                    var returnItems = {};
                    var key = items;
                    if (key in ss.storage) {
                        var val = ss.storage[key];
                        returnItems[key] = val;
                    }
                    panel.port.emit(responseEvent, returnItems);
                }
            });

            panel.show({
                position: button
            });
        }
    }
});

